@@ -16,11 +16,11 @@ Net::SMTP::Bulk - NonBlocking batch SMTP using Net::SMTP interface
 
 =head1 VERSION
 
-Version 0.21
+Version 0.22
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 
 
@@ -110,6 +110,8 @@ Auth - (Method 2 only) An ARRAY containing AUTH details. ( eg. ['AUTO','user','p
 
 Callbacks - You can supply callback functions on certain conditions, these conditions include: OPTIONAL
 
+Pipeline - (Method 2 only) Use pipelining for even quicker sending, makse sure server accepts pipelining. Pipelining should offer a good speed boost but may lead to more complex debugging, especially in mode 2 ( default: 0, 0 - Disabled with 4 round trips [normal], 1 - Pipelining with 2 roundtrips [faster], 2 - Pipelining with 1 roundtrip [fastest] ) OPTIONAL
+
 Method1:
 connect_pass,connect_fail,auth_pass,auth_fail,reconnect_pass,reconnect_fail,pass,fail,hang
 
@@ -135,6 +137,7 @@ pass - Email was accepted by server
 fail - This triggers if some sort of error happens, be it server did not recognize command or failed to connect. (I will most likely move fail to connect later on to connect_fail)
 hang - timeout has been reached for sending to email to user, email not sent
 global_hang - timeout has been reached for entire email sening operation
+read - callback on every line read
 
 The callback must return 1 it to follow proper proceedures. Other callback responses will be implemented later.
 
@@ -169,7 +172,7 @@ You can supply multiple hosts in an array.
 
 Other then the Caveats described above, it should he noted that there are missing functions for full SMTP compatibility.
 
-Missing fuctions include: HELO(EHLO is used), TURN, ATURN, SIZE, ETRN, PIPELINING, CHUNKING/BDAT, DSN, RSET, VRFY, HELP
+Missing fuctions include: HELO(EHLO is used), TURN, ATURN, SIZE, ETRN, CHUNKING/BDAT, DSN, RSET, VRFY, HELP
 
 These functions will be added as time goes on. If you need a certain function to have priority, request it .
 =cut
