@@ -8,15 +8,15 @@ use Encode;
 
 =head1 NAME
 
-Net::SMTP::Bulk::Coro - NonBlocking batch SMTP using Net::SMTP interface cia AnyEvent
+Net::SMTP::Bulk::AnyEvent - NonBlocking batch SMTP using Net::SMTP interface cia AnyEvent
 
 =head1 VERSION
 
-Version 0.21
+Version 0.22
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 =head2 new(%options, Hosts=>[\%options2,\%options3])
 
@@ -75,6 +75,7 @@ sub new {
                        port=>$new{Port}||25,
                        timeout=>$new{Timeout}||30,
                        secure=>$new{Secure}||0,
+                       pipe=>$new{Pipeline}||0,
 		       sleep=>{
                                hang=>0,
                                fail=>0,
@@ -358,6 +359,7 @@ sub _PREPARE {
         }
     
         $self->{secure}{ $new{Host} }=$new{Secure}||$self->{defaults}{secure};
+        $self->{pipe}{ $new{Host} }=$new{Pipeline}||$self->{defaults}{pipe};
         $self->{port}{ $new{Host} }||=$new{Port}||$self->{defaults}{port};
         $self->{helo}{ $new{Host} }=$new{Hello}||$self->{host}{ $new{Host} };
         $self->{timeout}{ $new{Host} }=$new{Timeout}||$self->{defaults}{timeout};
@@ -390,6 +392,7 @@ sub _PREPARE {
             $self->{objects}{ $new{Host} }{$t}={};
             $self->{queue}{ $new{Host} }{$t}=[];
             $self->{queue_size}{ $new{Host} }{$t}=0;
+            $self->{pipe_mode}{ $new{Host} }{$t}=[0,[],{}];
             $self->{stage}{ $new{Host} }{ $t }=['BEGIN',0];
             $self->{stats}{ $new{Host} }{ $t }={
                 queue => {
@@ -535,6 +538,26 @@ sub _HELO {
     my $k=shift;
     
     $self->_WRITE($k,'EHLO '.$self->{helo}{ $k->[0] });
+}
+
+sub _PIPE {
+    my $self=shift;
+    my $k=shift;
+    my $mode=shift;
+    
+    if ($self->{pipe}{ $k->[0] } > 0) {
+        
+        if ( $mode == 1 ) {
+            $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0]=$self->{pipe}{ $k->[0] };
+        }
+        
+        
+        
+    }
+    
+    
+    
+    
 }
 
 sub _STARTTLS {
@@ -623,9 +646,10 @@ sub _MAIL {
     #print "QUEUES($#{$self->{queue}{ $k->[0] }{ $k->[1] }})\n";
     
     if ( $self->{queue_size}{ $k->[0] }{ $k->[1] } == -1 ) {
-        
+      
       
       $self->_WRITE($k,'QUIT');
+	  
       $self->{fh}{ $k->[0] }{ $k->[1] }->destroy;
       $self->{cv}->end;
       
@@ -635,8 +659,22 @@ sub _MAIL {
     
     $self->{stage}{ $k->[0] }{ $k->[1] }=['MAIL',1];
     #$self->_WRITE($k,'MAIL FROM: '.$self->{data}{ $k->[0] }{ $k->[1] }{mail});
-    $self->_WRITE($k,'MAIL FROM: '.$self->{on_queue}{ $k->[0] }{ $k->[1] }{mail});
-    $self->{stage}{ $k->[0] }{ $k->[1] }=['MAIL',2];
+    
+    my $write = 'MAIL FROM: '.$self->{on_queue}{ $k->[0] }{ $k->[1] }{mail};
+    
+    if ( $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0] == 0 ) {
+        $self->_WRITE( $k, $write );
+		
+		$self->{stage}{ $k->[0] }{ $k->[1] }=['MAIL',2];
+    } else {
+	 
+        push( @{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] }, $write );
+		 $self->_TO($k);
+    }
+
+  
+    
+    
     }
 }
 
@@ -645,16 +683,40 @@ sub _TO {
     my $k=shift;
     $self->{stage}{ $k->[0] }{ $k->[1] }=['TO',1];
     #$self->_WRITE($k,'RCPT TO: '.$self->{data}{ $k->[0] }{ $k->[1] }{to});
-    $self->_WRITE($k,'RCPT TO: '.$self->{on_queue}{ $k->[0] }{ $k->[1] }{to});
-   
-    $self->{stage}{ $k->[0] }{ $k->[1] }=['TO',2];
+
+    my $write = 'RCPT TO: '.$self->{on_queue}{ $k->[0] }{ $k->[1] }{to};
+    
+    if ( $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0] == 0 ) {
+        $self->_WRITE( $k, $write );
+		$self->{stage}{ $k->[0] }{ $k->[1] }=['TO',2];
+    } else {
+        push( @{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] }, $write );
+		$self->_DATA($k);
+    }
+     
+    
 }
 
 sub _DATA {
     my $self=shift;
     my $k=shift;
+    
+    
     $self->{stage}{ $k->[0] }{ $k->[1] }=['DATA',1];
-    $self->_WRITE($k,'DATA');
+    
+    my $write = 'DATA';
+    
+    if ( $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0] == 0 ) {
+        $self->_WRITE( $k, $write );
+    } else {
+	    push( @{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] }, $write );
+		$write=join("\n",@{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] });
+		$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1]=[];
+	    $self->_WRITE($k, $write );
+
+    }
+    
+
     $self->{stage}{ $k->[0] }{ $k->[1] }=['DATA',2];
 }
 
@@ -663,10 +725,42 @@ sub _DATAEND {
     my $k=shift;
     $self->{stage}{ $k->[0] }{ $k->[1] }=['DATAEND',1];
     #$self->_WRITE($k,$self->{data}{ $k->[0] }{ $k->[1] }{data}."\r\n.");
-    $self->_WRITE($k,$self->{on_queue}{ $k->[0] }{ $k->[1] }{data}."\r\n.");
+
+    my $write = $self->{on_queue}{ $k->[0] }{ $k->[1] }{data}."\r\n.";
     
-    $self->{stage}{ $k->[0] }{ $k->[1] }=['DATAEND',2];
-    $self->{data}{ $k->[0] }{ $k->[1] }={};
+    if ( $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0] < 2 ) {
+	    $write = join("\n",@{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] })."\n".$write;
+		$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1]=[];
+        $self->_WRITE( $k, $write );
+
+		$self->{stage}{ $k->[0] }{ $k->[1] }=['DATAEND',2];
+		$self->{data}{ $k->[0] }{ $k->[1] }={};
+
+		} elsif ( $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0] == 2 ) {
+        
+
+		if ( $self->{queue_size}{ $k->[0] }{ $k->[1] } == 0 ) {
+			$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]=1;
+      	    $write = join("\n",@{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] })."\n".$write;
+			$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1]=[];
+			$self->_WRITE( $k, $write );
+			$self->{stage}{ $k->[0] }{ $k->[1] }=['DATAEND',2];
+			$self->{data}{ $k->[0] }{ $k->[1] }={};
+	    } else {
+		    $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]{to}=$self->{on_queue}{ $k->[0] }{ $k->[1] }{to};
+			$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]{queue}=$self->{queue}{ $k->[0] }{ $k->[1] };
+		
+			push( @{ $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[1] }, $write );
+			$self->{stage}{ $k->[0] }{ $k->[1] }=['MAIL',0];
+			$self->{data}{ $k->[0] }{ $k->[1] }={};
+			$self->_MAIL($k);
+		}
+
+		
+    }
+
+   
+
       #$self->{cv}->end;
 }
 
@@ -704,6 +798,7 @@ sub _READ {
             $self->{status_mode}{ $k->[0] }{ $k->[1] }=$2;
             $self->{status_text}{ $k->[0] }{ $k->[1] }=$3;
 
+            my $r_read=$self->_FUNC('read',$self,$k,0,$self->{queue}{ $k->[0] }{ $k->[1] });
 
             if ( $self->{status_code}{ $k->[0] }{ $k->[1] } == 250 and $self->{stage}{ $k->[0] }{ $k->[1] }[0] eq 'DATAEND' and $self->{stage}{ $k->[0] }{ $k->[1] }[1] == 2 ) {
                 #$self->{cv}->end;
@@ -720,7 +815,13 @@ sub _READ {
                 
                 #print "THREADS($self->{open_threads})\n";
                 $self->{stage}{ $k->[0] }{ $k->[1] }=['MAIL',0];
-            }
+            } elsif ( $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[0] == 2 and $self->{status_code}{ $k->[0] }{ $k->[1] } == 250 and $self->{stage}{ $k->[0] }{ $k->[1] }[0] eq 'DATA' and $self->{stage}{ $k->[0] }{ $k->[1] }[1] == 2 and $self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]{to} ne '') {
+
+				$self->{stats}{ $k->[0] }{ $k->[1] }{queue}{pass}++;
+                $self->_DEBUG($k,'Email : '.(++$self->{stats}{ $k->[0] }{ $k->[1] }{queue}{count}).' : '.$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]{to}.' : PASS',8) if $self->{debug} >= 1;
+                my $r=$self->_FUNC('pass',$self,$k,0,$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]{queue});
+				$self->{pipe_mode}{ $k->[0] }{ $k->[1] }[2]={};
+			}
             
             if ( $self->{status_code}{ $k->[0] }{ $k->[1] } == 220 and ($self->{stage}{ $k->[0] }{ $k->[1] }[0] eq 'BEGIN' or $self->{stage}{ $k->[0] }{ $k->[1] }[0] eq 'HELO') ) {
                 $self->{stage}{ $k->[0] }{ $k->[1] }=['HELO',0];
@@ -759,9 +860,12 @@ sub _READ {
                     if ( ($self->{secure}{ $k->[0] } == 3 or $self->{secure}{ $k->[0] } == 4) and exists($self->{header}{ $k->[0] }{ $k->[1] }{starttls})  ) {
                        $self->_STARTTLS($k);
                     } elsif ( ($self->{auth}{ $k->[0] }{ $k->[1] }[0]||'0') ne '0' ) {
+                        $self->_PIPE($k,1);
+                        
                         $self->_AUTH($k);
  
                     } else {
+                         $self->_PIPE($k,1);
                          $self->{stage}{ $k->[0] }{ $k->[1] }=['MAIL',0];
                        
                     }
@@ -779,7 +883,7 @@ sub _READ {
                 $self->_MAIL($k);
                 
             } elsif ( $self->{stage}{ $k->[0] }{ $k->[1] }[0] eq 'MAIL' and $self->{stage}{ $k->[0] }{ $k->[1] }[1] == 2  ) {
-                
+            
                 if ($self->{status_code}{ $k->[0] }{ $k->[1] } == 250) {
                     $self->{stage}{ $k->[0] }{ $k->[1] }=['TO',0];
                     $self->_TO($k);
@@ -798,7 +902,7 @@ sub _READ {
                 
             } elsif ( $self->{stage}{ $k->[0] }{ $k->[1] }[0] eq 'DATA' and $self->{stage}{ $k->[0] }{ $k->[1] }[1] == 2  ) {
                 
-                if ($self->{status_code}{ $k->[0] }{ $k->[1] } == 354) {
+                if ($self->{status_code}{ $k->[0] }{ $k->[1] } == 354 ) {
                     $self->{stage}{ $k->[0] }{ $k->[1] }=['DATAEND',0];
                     $self->_DATAEND($k);
                     
